
# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(BuildFileAddress(src/python/twitter/common/collections/BUILD, collections))

from setuptools import setup

setup(**
{   'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python'],
    'description': 'twitter.common collections library.',
    'install_requires': ['twitter.common.lang==0.3.11'],
    'license': 'Apache License, Version 2.0',
    'name': 'twitter.common.collections',
    'namespace_packages': ['twitter', 'twitter.common'],
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': ['twitter', 'twitter.common', 'twitter.common.collections'],
    'url': 'https://github.com/twitter/commons',
    'version': '0.3.11',
    'zip_safe': True}
)
